@@ -26,153 +26,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: channels.h,v $
- * Revision 1.3  2008/05/23 11:19:02  willamowius
- * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
- *
- * Revision 1.2  2008/04/25 01:17:36  shorne
- * Added missing virtual information off H323_ExternalRTPChannel
- *
- * Revision 1.1  2007/08/06 20:50:48  shorne
- * First commit of h323plus
- *
- * Revision 1.42.4.1  2007/05/23 06:58:01  shorne
- * Nat Support for EP's nested behind same NAT
- *
- * Revision 1.42  2005/11/30 13:05:00  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 1.41  2004/07/03 05:47:45  rjongbloed
- * Added virtual function for determining RTP payload type used in an H.323 channel,
- *    also some added bullet proofing for exception conditions, thanks Guilhem Tardy
- *
- * Revision 1.40  2002/09/16 01:14:15  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 1.39  2002/09/03 06:19:36  robertj
- * Normalised the multi-include header prevention ifdef/define symbol.
- *
- * Revision 1.38  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.37  2002/06/25 08:30:08  robertj
- * Changes to differentiate between stright G.723.1 and G.723.1 Annex A using
- *   the OLC dataType silenceSuppression field so does not send SID frames
- *   to receiver codecs that do not understand them.
- *
- * Revision 1.36  2002/05/23 04:53:54  robertj
- * Added function to remove a filter from logical channel.
- *
- * Revision 1.35  2002/05/10 05:46:46  robertj
- * Added session ID to the data logical channel class.
- *
- * Revision 1.34  2002/05/02 07:56:24  robertj
- * Added automatic clearing of call if no media (RTP data) is transferred in a
- *   configurable (default 5 minutes) amount of time.
- *
- * Revision 1.33  2002/05/02 06:28:50  robertj
- * Fixed problem with external RTP channels not fast starting.
- *
- * Revision 1.32  2002/04/17 05:56:28  robertj
- * Added trace output of H323Channel::Direction enum.
- *
- * Revision 1.31  2002/02/09 04:39:01  robertj
- * Changes to allow T.38 logical channels to use single transport which is
- *   now owned by the OpalT38Protocol object instead of H323Channel.
- *
- * Revision 1.30  2002/02/05 08:13:20  robertj
- * Added ability to not have addresses when external RTP channel created.
- *
- * Revision 1.29  2002/01/22 22:48:21  robertj
- * Fixed RFC2833 support (transmitter) requiring large rewrite
- *
- * Revision 1.28  2002/01/17 07:04:57  robertj
- * Added support for RFC2833 embedded DTMF in the RTP stream.
- *
- * Revision 1.27  2002/01/17 00:10:37  robertj
- * Fixed double copy of rtpPayloadType in RTP channel, caused much confusion.
- *
- * Revision 1.26  2002/01/14 05:18:16  robertj
- * Fixed typo on external RTP channel constructor.
- *
- * Revision 1.25  2002/01/10 05:13:50  robertj
- * Added support for external RTP stacks, thanks NuMind Software Systems.
- *
- * Revision 1.24  2001/10/23 02:18:06  dereks
- * Initial release of CU30 video codec.
- *
- * Revision 1.23  2001/07/24 02:26:53  robertj
- * Added start for handling reverse channels.
- *
- * Revision 1.22  2001/07/17 04:44:29  robertj
- * Partial implementation of T.120 and T.38 logical channels.
- *
- * Revision 1.21  2001/02/09 05:16:24  robertj
- * Added #pragma interface for GNU C++.
- *
- * Revision 1.20  2001/01/25 07:27:14  robertj
- * Major changes to add more flexible OpalMediaFormat class to normalise
- *   all information about media types, especially codecs.
- *
- * Revision 1.19  2000/10/19 04:05:20  robertj
- * Added compare function for logical channel numbers, thanks Yuriy Ershov.
- *
- * Revision 1.18  2000/09/22 01:35:02  robertj
- * Added support for handling LID's that only do symmetric codecs.
- *
- * Revision 1.17  2000/08/31 08:15:32  robertj
- * Added support for dynamic RTP payload types in H.245 OpenLogicalChannel negotiations.
- *
- * Revision 1.16  2000/08/21 02:50:19  robertj
- * Fixed race condition if close call just as slow start media channels are opening.
- *
- * Revision 1.15  2000/06/15 01:46:13  robertj
- * Added channel pause (aka mute) functions.
- *
- * Revision 1.14  2000/05/18 11:53:33  robertj
- * Changes to support doc++ documentation generation.
- *
- * Revision 1.13  2000/05/02 04:32:23  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.12  1999/12/23 23:02:34  robertj
- * File reorganision for separating RTP from H.323 and creation of LID for VPB support.
- *
- * Revision 1.11  1999/11/19 09:06:30  robertj
- * Changed to close down logical channel if get a transmit codec error.
- *
- * Revision 1.10  1999/11/06 05:37:44  robertj
- * Complete rewrite of termination of connection to avoid numerous race conditions.
- *
- * Revision 1.9  1999/09/08 04:05:48  robertj
- * Added support for video capabilities & codec, still needs the actual codec itself!
- *
- * Revision 1.8  1999/08/31 12:34:18  robertj
- * Added gatekeeper support.
- *
- * Revision 1.7  1999/07/09 06:09:49  robertj
- * Major implementation. An ENORMOUS amount of stuff added everywhere.
- *
- * Revision 1.6  1999/06/14 05:15:55  robertj
- * Changes for using RTP sessions correctly in H323 Logical Channel context
- *
- * Revision 1.5  1999/06/13 12:41:14  robertj
- * Implement logical channel transmitter.
- * Fixed H245 connect on receiving call.
- *
- * Revision 1.4  1999/06/09 05:26:19  robertj
- * Major restructuring of classes.
- *
- * Revision 1.3  1999/06/06 06:06:36  robertj
- * Changes for new ASN compiler and v2 protocol ASN files.
- *
- * Revision 1.2  1999/04/26 06:14:46  craigs
- * Initial implementation for RTP decoding and lots of stuff
- * As a whole, these changes are called "First Noise"
- *
- * Revision 1.1  1999/01/16 01:31:07  robertj
- * Initial revision
+ * $Id: channels.h,v 1.12 2014/01/16 02:27:08 shorne Exp $
  *
  */
 
@@ -293,6 +147,10 @@ class H323Channel : public PObject
      */
     virtual unsigned GetSessionID() const;
 
+    /**Set the session number of the Channel
+     */
+    virtual void SetSessionID(unsigned id);
+
     /**Set the initial bandwidth for the channel.
        This calculates the initial bandwidth required by the channel and
        returns TRUE if the connection can support this bandwidth.
@@ -408,6 +266,11 @@ class H323Channel : public PObject
        Typically, used to indicate a problem in the received video stream.
     */
     void SendMiscCommand(unsigned command);
+    
+    /**
+       Send a flow control request to the remote
+    */
+    void SendFlowControlRequest(long restriction);
   //@}
 
   /**@name Member variable access */
@@ -418,7 +281,7 @@ class H323Channel : public PObject
 
     /**Set the number of the channel.
      */
-    void SetNumber(const H323ChannelNumber & num) { number = num; }
+    virtual void SetNumber(const H323ChannelNumber & num) { number = num; }
 
     /**Get the number of the reverse channel (if present).
      */
@@ -467,6 +330,31 @@ class H323Channel : public PObject
     void SetPause(
       PBoolean pause   ///< New pause flag
     ) { paused = pause; }
+
+   /** Is Media Tunneled
+     */
+    PBoolean IsMediaTunneled() const { return mediaTunneled; }
+
+    /**Replace the capability
+      */
+    virtual void ReplaceCapability(const H323Capability & cap);
+
+    /**Set Associated Channel
+        This can be used to set the associated encryption channel wrapper
+      */
+    virtual void SetAssociatedChannel(H323Channel * /*channel*/) {}
+
+    /**Get the active payload type used by this channel.
+       This will use the dynamic payload type configured for the channel, or
+       the fixed payload type defined by the media format.
+       */
+    virtual RTP_DataFrame::PayloadTypes GetRTPPayloadType() const { return RTP_DataFrame::IllegalPayloadType;  }
+
+    /**Set the dynamic payload type used by this channel.
+      */
+    virtual PBoolean SetDynamicRTPPayloadType(
+      int /*newType*/  ///< New RTP payload type number
+      )  { return false; }
   //@}
 
   protected:
@@ -481,13 +369,14 @@ class H323Channel : public PObject
     PBoolean                   opened;
     PBoolean                   paused;
     PBoolean                   terminating;
+    PBoolean                   mediaTunneled;
 
   private:
     unsigned bandwidthUsed;
 };
 
 
-PLIST(H323LogicalChannelList, H323Channel);
+H323LIST(H323LogicalChannelList, H323Channel);
 
 
 
@@ -575,6 +464,7 @@ class H323BidirectionalChannel : public H323Channel
 
 /**This class is for encpsulating the IETF Real Time Protocol interface.
  */
+class H245_DataType;
 class H323_RealTimeChannel : public H323UnidirectionalChannel
 {
   PCLASSINFO(H323_RealTimeChannel, H323UnidirectionalChannel);
@@ -639,11 +529,18 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
       H245_H2250LogicalChannelParameters & param  ///< Open PDU to send.
     ) const = 0;
 
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual PBoolean OnSendingAltPDU(
-	  H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-	) const = 0;
+    /**Fill out the OpenLogicalChannel PDU for the particular channel type.
+     */
+    virtual PBoolean OnSendingPDU(
+      H245_DataType & /*dataType*/,                 ///< DataType Information
+      H245_H2250LogicalChannelParameters & param    ///< Open PDU to send.
+    ) const { return OnSendingPDU(param); }
+    
+    /**Alternate RTP port information for Same NAT
+      */
+    virtual PBoolean OnSendingAltPDU(
+      H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
+    ) const = 0;
 
     /**This is called when request to create a channel is received from a
        remote machine and is about to be acknowledged.
@@ -652,12 +549,6 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
       H245_H2250LogicalChannelAckParameters & param ///< Acknowledgement PDU
     ) const = 0;
 
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual void OnSendOpenAckAlt(
-	  H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-	) const = 0;
-
     /**This is called after a request to create a channel occurs from the
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
@@ -669,11 +560,11 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
       unsigned & errorCode                              ///< Error on failure
     ) = 0;
 
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual PBoolean OnReceivedAltPDU(
-	  const H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-	) = 0;
+    virtual PBoolean OnReceivedPDU(
+      const H245_DataType & /*dataType*/,               ///< DataType Information
+      const H245_H2250LogicalChannelParameters & param, ///< Acknowledgement PDU
+      unsigned & errorCode                              ///< Error on failure
+     ) { return OnReceivedPDU(param, errorCode); }
 
     /**This is called after a request to create a channel occurs from the
        local machine via the H245LogicalChannelDict::Open() function, and
@@ -685,11 +576,11 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
       const H245_H2250LogicalChannelAckParameters & param ///< Acknowledgement PDU
     ) = 0;
 
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual PBoolean OnReceivedAckAltPDU(
-	  const H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-	) = 0;
+    /**Alternate RTP port information for Same NAT
+      */
+    virtual PBoolean OnReceivedAckAltPDU(
+      const H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
+    ) = 0;
 
     /**Get the active payload type used by this channel.
        This will use the dynamic payload type configured for the channel, or
@@ -745,6 +636,10 @@ class H323_RTPChannel : public H323_RealTimeChannel
      */
     virtual unsigned GetSessionID() const;
 
+    /**Set the session number of the Channel
+     */
+    virtual void SetSessionID(unsigned id);
+
     /**Open the channel.
       */
     virtual PBoolean Open();
@@ -764,6 +659,25 @@ class H323_RTPChannel : public H323_RealTimeChannel
        (eg an RTP_session).
       */
     virtual void Transmit();
+
+    /**Handle UniChannel Back Probe.
+
+       When receiving a Unidirectional media stream send back to the
+       originator empty RTPFrames to open necessary pinholes in NAT.
+      */
+    virtual void SendUniChannelBackProbe();
+
+    /** Read a DataFrame 
+      */
+    virtual PBoolean ReadFrame(DWORD & rtpTimestamp,     ///< TimeStamp
+                               RTP_DataFrame & frame     ///< RTP data frame
+        );
+
+    /** Write a DataFrame 
+      */
+    virtual PBoolean WriteFrame(RTP_DataFrame & frame     ///< RTP data frame
+        );
+
   //@}
 
   /**@name Overrides from class H323_RealTimeChannel */
@@ -774,11 +688,11 @@ class H323_RTPChannel : public H323_RealTimeChannel
       H245_H2250LogicalChannelParameters & param  ///< Open PDU to send.
     ) const;
 
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual PBoolean OnSendingAltPDU(
-	  H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-	) const;
+    /**Alternate RTP port information for Same NAT
+      */
+    virtual PBoolean OnSendingAltPDU(
+      H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
+    ) const;
 
     /**This is called when request to create a channel is received from a
        remote machine and is about to be acknowledged.
@@ -787,12 +701,6 @@ class H323_RTPChannel : public H323_RealTimeChannel
       H245_H2250LogicalChannelAckParameters & param ///< Acknowledgement PDU
     ) const;
 
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual void OnSendOpenAckAlt(
-	  H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-	) const;
-
     /**This is called after a request to create a channel occurs from the
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
@@ -804,12 +712,6 @@ class H323_RTPChannel : public H323_RealTimeChannel
       unsigned & errorCode                              ///< Error on failure
     );
 
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual PBoolean OnReceivedAltPDU(
-	  const H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-	);
-
     /**This is called after a request to create a channel occurs from the
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
@@ -820,11 +722,11 @@ class H323_RTPChannel : public H323_RealTimeChannel
       const H245_H2250LogicalChannelAckParameters & param ///< Acknowledgement PDU
     );
 
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual PBoolean OnReceivedAckAltPDU(
-	  const H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
-	);
+    /**Alternate RTP port information for Same NAT
+      */
+    virtual PBoolean OnReceivedAckAltPDU(
+      const H245_ArrayOf_GenericInformation & alternate  ///< Alternate RTP ports
+    );
   //@}
 
     void AddFilter(
@@ -841,11 +743,14 @@ class H323_RTPChannel : public H323_RealTimeChannel
     RTP_Session      & rtpSession;
     H323_RTP_Session & rtpCallbacks;
 
-    PLIST(FilterList, PNotifier);
+    H323LIST(FilterList, PNotifier);
     FilterList filters;
     PMutex     filterMutex;
 
     PTimeInterval silenceStartTick;
+
+    unsigned rec_written;
+    PBoolean rec_ok;
 };
 
 
@@ -854,6 +759,7 @@ class H323_RTPChannel : public H323_RealTimeChannel
 /**This class is for encpsulating the IETF Real Time Protocol interface as used
 by a remote host.
  */
+class H323SecureChannel;
 class H323_ExternalRTPChannel : public H323_RealTimeChannel
 {
   PCLASSINFO(H323_ExternalRTPChannel, H323_RealTimeChannel);
@@ -960,46 +866,26 @@ class H323_ExternalRTPChannel : public H323_RealTimeChannel
       const H245_H2250LogicalChannelAckParameters & param ///< Acknowledgement PDU
     );
 
-	/**Sending alternate RTP ports if behind same NAT
-	  */
-	virtual PBoolean OnSendingAltPDU(
-	  H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
-	  ) const  {  return FALSE;  }
-
-	/**This is called if the call parties are detected behind the same NAT
-	   Use this to set alternate internal LAN ports
-	  */
-	virtual void OnSendOpenAckAlt(
-	  H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
-	) const {}
-
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual PBoolean OnReceivedAltPDU(
-	  const H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
-	)   { return FALSE;  }
-
-	/**Alternate RTP port information for Same NAT
-	  */
-	virtual PBoolean OnReceivedAltPDU(
-	   H323_RTPChannel & /*channel*/,                                       ///< Channel using this session.
-	  const H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
-	  )  { return FALSE;  }
+    /**Sending alternate RTP ports if behind same NAT
+      */
+    virtual PBoolean OnSendingAltPDU(
+      H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
+      ) const  {  return FALSE;  }
 
       /**This is called after a request to create a channel occurs from the
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
      */
-	virtual PBoolean OnReceivedAckAltPDU(
+    virtual PBoolean OnReceivedAckAltPDU(
       H323_RTPChannel & /*channel*/,                                        ///< Channel using this session.
-	  const H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
-	  )   { return FALSE; }
+      const H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
+      )   { return FALSE; }
 
-	/**Alternate RTP port information for Same NAT
-	  */
-  	virtual PBoolean OnReceivedAckAltPDU(
-	  const H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
-	 )    { return FALSE; };
+    /**Alternate RTP port information for Same NAT
+      */
+    virtual PBoolean OnReceivedAckAltPDU(
+      const H245_ArrayOf_GenericInformation & /*alternate*/  ///< Alternate RTP ports
+     )    { return FALSE; };
   //@}
 
     void SetExternalAddress(
@@ -1015,6 +901,13 @@ class H323_ExternalRTPChannel : public H323_RealTimeChannel
       WORD & dataPort
     ) const;
 
+    // Encryption Support
+#ifdef H323_H235
+    void AttachSecureChannel(H323SecureChannel * channel);
+#endif
+    PBoolean OnReadFrame(RTP_DataFrame & frame);
+    PBoolean OnWriteFrame(RTP_DataFrame & frame);
+
   protected:
     unsigned             sessionID;
     H323TransportAddress externalMediaAddress;
@@ -1023,6 +916,10 @@ class H323_ExternalRTPChannel : public H323_RealTimeChannel
     H323TransportAddress remoteMediaControlAddress;
 
     PBoolean isRunning;
+
+#ifdef H323_H235
+    H323SecureChannel * secChannel;
+#endif
 };
 
 
