@@ -26,7 +26,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Id: h323pdu.h,v 1.10 2012/05/17 09:48:15 shorne Exp $
+ * $Id: h323pdu.h,v 1.14 2014/01/09 12:12:02 shorne Exp $
  *
  */
 
@@ -269,21 +269,28 @@ class H323SignalPDU : public H225_H323_UserInformation
     );
 
 #ifdef H323_H460
-	/** When sending the H460 message in the Setup PDU you have to ensure
-	    the ARQ is received first then add the Fields to the Setup PDU
-		so we require a call back
-	  */
-	void InsertH460Setup(const H323Connection & connection, H225_Setup_UUIE & setup);
+    /** When sending the H460 message in the Setup PDU you have to ensure
+        the ARQ is received first then add the Fields to the Setup PDU
+       so we require a call back
+      */
+    PBoolean InsertH460Generic(const H323Connection & connection);
+
+    /** When sending the H460 message in the Setup PDU you have to ensure
+        the ARQ is received first then add the Fields to the Setup PDU
+        so we require a call back
+      */
+    void InsertH460Setup(const H323Connection & connection, H225_Setup_UUIE & setup);
 #endif
 
-#ifndef DISABLE_CALLAUTH
-	/** When sending the Setup PDU you have to ensure
-	    the ARQ is received first then add the cryptoFields to the Setup PDU
-		so we require a call back
-	  */
+    /** When sending the Setup PDU you have to ensure
+        the ARQ is received first then add the cryptoFields to the Setup PDU
+        so we require a call back
+     */
     void InsertCryptoTokensSetup(const H323Connection & connection, H225_Setup_UUIE & setup);
-#endif
 
+#ifdef H323_H46017
+    void LoadTunneledQ931(const Q931 & q931);
+#endif
 
   protected:
     // Even though we generally deal with the H323 protocol (H225) it is
@@ -472,11 +479,20 @@ class H323RasPDU : public H225_RasMessage, public H323TransactionPDU
 // Language support
 
 void H323SetLanguage(const PString & lang, PASN_IA5String & asn);
-PBoolean H323SetLanguages(const PStringList & lang, H225_Setup_UUIE_language & asn);
-PBoolean H323SetLanguages(const PStringList & lang, H225_Connect_UUIE_language & asn);
 void H323GetLanguage(PStringList & lang, const PASN_IA5String & asn);
-PBoolean H323GetLanguages(PStringList & lang, const H225_Setup_UUIE_language & asn);
-PBoolean H323GetLanguages(PStringList & lang, const H225_Connect_UUIE_language & asn);
+
+#define H323LANGUAGEPDU(pdu) \
+    PBoolean H323SetLanguages(const PStringList & lang, pdu##_language & asn); \
+    PBoolean H323GetLanguages(PStringList & lang, const pdu##_language & asn);
+
+H323LANGUAGEPDU(H225_Setup_UUIE);
+H323LANGUAGEPDU(H225_Connect_UUIE);
+H323LANGUAGEPDU(H225_RegistrationRequest);
+H323LANGUAGEPDU(H225_RegistrationConfirm);
+H323LANGUAGEPDU(H225_AdmissionConfirm);
+H323LANGUAGEPDU(H225_LocationRequest);
+H323LANGUAGEPDU(H225_LocationConfirm);
+
 
 PString H323GetDisplayName(const H225_ArrayOf_DisplayName & asn);
 PBoolean H323SetDisplayName(const PStringList & alias, const PStringList & lang, H225_ArrayOf_DisplayName & asn);
