@@ -35,7 +35,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Id: h4601.h,v 1.29 2012/06/18 16:48:16 shorne Exp $
+ * $Id: h4601.h,v 1.34 2014/01/05 21:53:43 shorne Exp $
  *
  *
  */
@@ -488,7 +488,7 @@ class H460_FeatureTable : public H225_ArrayOf_EnumeratedParameter
     /** ParameterCount
         Number of Feature Parameters in the Parameter List.
     */
-    int ParameterCount() { return GetSize(); };
+    PINDEX ParameterCount() { return GetSize(); };
 
     /** ParameterIsUnique
         return TRUE if there is only 1 instance of
@@ -600,13 +600,12 @@ class H460_Feature : public H225_FeatureDescriptor
 
 
     enum {
-      FeatureBase      =4,        ///< Create Startup use only Startup
-      FeatureBaseAll   =5,        ///< Create Startup use RAS and Signal
-      FeatureBaseRas   =6,        ///< Create Startup use RAS
-      FeatureBaseSignal=7,          ///< Create Base use Signal
-      FeatureRas       =8,        ///< Create Registering GK
-      FeaturePresence  =10,       ///< Create Call Setup/Advertise in Presence
-      FeatureSignal    =16        ///< Create Call Setup
+      FeatureBaseAll   =0,        ///< AutoCreate Startup share instance with RAS and Signal
+      FeatureBaseClone =2,        ///< AutoCreate Startup clone instance in RAS and Signal
+      FeatureBase      =4,        ///< AutoCreate Startup only (not used in RAS or Call Signalling)
+      FeatureRas       =8,        ///< AutoCreate RAS Only 
+      FeatureSignal    =16,       ///< AutoCreate Call Signalling Only
+      FeaturePresence  =64        ///< Manual Create in endpoint as required
     } FeatureInstance;
 
   //@}
@@ -765,7 +764,7 @@ class H460_Feature : public H225_FeatureDescriptor
     /** Get the purpose of the the Feature whether for Signalling,RAS or both. This determines
         when the class is instantized
       */
-    static int GetPurpose()    { return FeatureBase; };
+    static int GetPurpose()    { return FeatureBaseAll; };
 
     /** Get the Feature Identifier
       */
@@ -1173,6 +1172,26 @@ class H460_FeatureSet : public PObject
 
 /////////////////////////////////////////////////////////////////////
 
+#if PTLIB_VER >= 2130
+
+PCREATE_PLUGIN_DEVICE(H460_Feature);
+#define H460_FEATURE_EX(name, extra) \
+PCREATE_PLUGIN(name, H460_Feature, H460_Feature##name, PPlugin_H460_Feature, \
+    virtual PStringArray GetDeviceNames(P_INT_PTR userData) const { \
+        return (userData == 1 ? H460_Feature##name::GetIdentifier() : H460_Feature##name::GetFeatureFriendlyName()); } \
+    virtual bool  ValidateDeviceName(const PString & deviceName, int userData) const { \
+        PStringArray devices = H460_Feature##name::GetFeatureName(); \
+        if (deviceName == devices[0]) { \
+             int use = H460_Feature##name::GetPurpose(); \
+             if (use == H460_Feature::FeatureBaseAll || use == H460_Feature::FeatureBaseClone || use == userData) \
+                return true; \
+        } \
+        return false; \
+    }; \
+extra)
+#define H460_FEATURE(name) H460_FEATURE_EX(name, )
+#else
+
 template <class className> class H460PluginServiceDescriptor : public PDevicePluginServiceDescriptor
 {
   public:
@@ -1187,19 +1206,21 @@ template <class className> class H460PluginServiceDescriptor : public PDevicePlu
     virtual bool  ValidateDeviceName(const PString & deviceName, int userData) const 
     { 
          PStringArray devices = className::GetFeatureName(); 
-         if ((deviceName == devices[0]) &&
-             (className::GetPurpose() >= userData) && 
-             (className::GetPurpose() < userData*2)) {
-
+         if (deviceName == devices[0]) {
+             int use = className::GetPurpose();
+             if (use == H460_Feature::FeatureBaseAll || use == H460_Feature::FeatureBaseClone || use == userData) 
                 return true; 
-         } else
-                return false;
-    } 
+         } 
+         return false;
+    }
 };
 
 #define H460_FEATURE(name)    \
 static H460PluginServiceDescriptor<H460_Feature##name> H460_Feature##name##_descriptor; \
-PCREATE_PLUGIN_STATIC(name, H460_Feature, &H460_Feature##name##_descriptor); \
+PCREATE_PLUGIN(name, H460_Feature, &H460_Feature##name##_descriptor);
+#endif
+
+
 
 /////////////////////////////////////////////////////////////////////
 
