@@ -36,7 +36,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Id: h46018_h225.h,v 1.28 2012/12/10 17:56:08 shorne Exp $
+ * $Id: h46018_h225.h,v 1.40 2014/01/08 06:25:15 shorne Exp $
  *
  */
 
@@ -166,12 +166,9 @@ class H46018Transport  : public H323TransportTCP
      PTimeInterval ReadTimeOut;
      PSyncPoint ReadMutex;
 
-     H46018Handler * Feature;
-
      PBoolean   isConnected;
      PBoolean   remoteShutDown;
-     PBoolean    closeTransport;
-     
+     PBoolean   closeTransport;
 };
 
 
@@ -201,6 +198,8 @@ class H46018Handler : public PObject
 #ifdef H323_H46024A
     void H46024ADirect(bool reply, const PString & token);
 #endif
+
+    void SetTransportSecurity(const H323TransportSecurity & callSecurity);
         
   protected:    
     H323EndPoint & EP;
@@ -218,6 +217,8 @@ class H46018Handler : public PObject
     PThread * SocketCreateThread;
     PDECLARE_NOTIFIER(PThread, H46018Handler, SocketThread);
     PBoolean m_h46018inOperation;
+
+    H323TransportSecurity m_callSecurity;
 };
 
 
@@ -228,9 +229,9 @@ typedef map<PString, unsigned> muxPortMap;
 class H46019MultiplexSocket;
 #endif
 
-class PNatMethod_H46019  : public PNatMethod
+class PNatMethod_H46019  : public H323NatMethod
 {
-    PCLASSINFO(PNatMethod_H46019,PNatMethod);
+    PCLASSINFO(PNatMethod_H46019,H323NatMethod);
 
   public:
     /**@name Construction */
@@ -344,12 +345,14 @@ class PNatMethod_H46019  : public PNatMethod
     /**  GetMethodName
         Get the NAT method name 
     */
-#if PTLIB_VER > 2120
-   static PString GetNatMethodName() { return "GnuGk"; }
+#if PTLIB_VER >= 2130
+   virtual PCaselessString GetMethodName() const { return "H46019"; }
+#elif PTLIB_VER > 2120
+   static PString GetNatMethodName() { return "H46019"; }
    virtual PString GetName() const
             { return GetNatMethodName(); }
 #else
-    static PStringList GetNatMethodName() {  return PStringArray("GnuGk"); };
+    static PStringList GetNatMethodName() {  return PStringArray("H46019"); };
     virtual PString GetName() const
             { return GetNatMethodName()[0]; }
 #endif
@@ -387,8 +390,14 @@ class PNatMethod_H46019  : public PNatMethod
             const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),WORD localPort = 0)  { return false; }
     virtual void SetCredentials(const PString &, const PString &, const PString &) {}
 protected:
+#if PTLIB_VER < 2130
     virtual NatTypes InternalGetNatType(bool, const PTimeInterval &) { return UnknownNat; }
 #endif
+#if PTLIB_VER >= 2120
+    virtual PNATUDPSocket * InternalCreateSocket(Component component, PObject * context)  { return NULL; }
+    virtual void InternalUpdate() {};
+#endif
+#endif
 
 
   protected:
@@ -443,7 +452,7 @@ struct  H46019MultiPacket {
 
 typedef std::queue<H46019MultiPacket> H46019MultiQueue;
 
-class H46019MultiplexSocket : public PUDPSocket
+class H46019MultiplexSocket : public H323UDPSocket
 {
   PCLASSINFO(H46019MultiplexSocket, PUDPSocket);
 
@@ -500,9 +509,9 @@ class H46019MultiplexSocket : public PUDPSocket
 };
 #endif
 
-class H46019UDPSocket : public PUDPSocket
+class H46019UDPSocket : public H323UDPSocket
 {
-    PCLASSINFO(H46019UDPSocket, PUDPSocket);
+    PCLASSINFO(H46019UDPSocket, H323UDPSocket);
   public:
     /**@name Construction/Deconstructor */
     /** create a UDP Socket Fully Nat Supported
@@ -649,7 +658,7 @@ class H46019UDPSocket : public PUDPSocket
 #if defined(H323_H46024A) || defined(H323_H46024B)
 
     enum  probe_state {
-        e_notRequired,        ///< Polling has not started
+        e_notRequired,        ///< Polling not required
         e_initialising,        ///< We are initialising (local set but remote not)
         e_idle,                ///< Idle (waiting for first packet from remote)
         e_probing,            ///< Probing for direct route
@@ -659,6 +668,8 @@ class H46019UDPSocket : public PUDPSocket
         e_direct            ///< we are going direct to detected address
     };
 
+    PString ProbeState(probe_state state);
+
     struct probe_packet {
         PUInt16b    Length;        // Length
         PUInt32b    SSRC;        // Time Stamp
@@ -719,7 +730,6 @@ class H46019UDPSocket : public PUDPSocket
     void ProbeReceived(bool probe, const PIPSocket::Address & addr, WORD & port);
     void SetProbeState(probe_state newstate);
     int GetProbeState() const;
-    probe_state m_state;
     PMutex probeMutex;
 #endif
 
@@ -765,11 +775,16 @@ private:
     PIPSocket::Address m_altAddr;  
 	WORD m_altPort;                                           ///< supplied remote Address (as supplied in Generic Information)
 	unsigned m_altMuxID;
+
+#ifdef H323_H46024A
+    probe_state m_state;
+#endif
+
 #ifdef H323_H46024B
-    // H46024 Annex B support
     PBoolean    m_h46024b;
 #endif
 
+	PAdaptiveDelay selectBlock;
     bool rtpSocket;
 
 };
