@@ -33,7 +33,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Id: h460_std18.h,v 1.9 2012/02/02 13:11:54 shorne Exp $
+ * $Id: h460_std18.h,v 1.11 2013/12/12 12:21:11 shorne Exp $
  *
  *
  */
@@ -67,13 +67,15 @@ public:
     // Universal Declarations Every H460 Feature should have the following
     virtual void AttachEndPoint(H323EndPoint * _ep);
 
-    static PStringArray GetFeatureName() { return PStringArray("Std18"); };
-    static PStringArray GetFeatureFriendlyName() { return PStringArray("NatTraversal-H.460.18"); };
-    static int GetPurpose()	{ return FeatureRas; };
-	static PStringArray GetIdentifier() { return PStringArray("18"); };
+    static PStringArray GetFeatureName() { return PStringArray("Std18"); }
+    static PStringArray GetFeatureFriendlyName() { return PStringArray("NatTraversal-H.460.18"); }
+    static int GetPurpose()	{ return FeatureRas; }
+	static PStringArray GetIdentifier() { return PStringArray("18"); }
 
 	virtual PBoolean CommonFeature() { return isEnabled; }
 
+    void SetTransportSecurity(const H323TransportSecurity & callSecurity);
+
     /////////////////////
     // H.460.18 Messages
     virtual PBoolean OnSendGatekeeperRequest(H225_FeatureDescriptor & pdu);
@@ -125,6 +127,8 @@ public:
 
 	virtual PBoolean CommonFeature() { return remoteSupport; }
 
+    virtual PBoolean FeatureAdvertised(int mtype);
+
     /////////////////////
     // H.460.19 Messages
     virtual PBoolean OnSendSetup_UUIE(H225_FeatureDescriptor & pdu);
