@@ -34,7 +34,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Id: ptlib_extras.h,v 1.27 2012/10/26 11:51:03 shorne Exp $
+ * $Id: ptlib_extras.h,v 1.42 2013/12/30 06:13:50 shorne Exp $
  *
  */
 
@@ -46,9 +46,18 @@
 #include <ptclib/delaychan.h>
 #include <algorithm>
 #include <queue>
+#include <limits>
+
+#include <ptclib/pnat.h>
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////
 
+#if PTLIB_VER < 2120
+#define H323_INT INT
+#else
+#define H323_INT P_INT_PTR
+#endif
+
 #ifndef H323_STLDICTIONARY
 
 #define H323Dictionary  PDictionary
@@ -271,6 +280,7 @@ template <class K, class D> class PSTLDictionary : public PObject,
 
           PAssert(ref < this->size(), psprintf("Index out of Bounds ref: %u sz: %u",ref,this->size()));
           typename std::map< unsigned, std::pair<K, D*>,PSTLSortOrder>::const_iterator i = this->find(ref);
+          PAssert(i != this->end(), psprintf("Item %u not found in collection sz: %u",ref,this->size()));
           return *(i->second.second);   
       };
 
@@ -282,12 +292,13 @@ template <class K, class D> class PSTLDictionary : public PObject,
 
           PAssert(ref < this->size(), psprintf("Index out of Bounds ref: %u sz: %u",ref,this->size()));
           typename std::map< unsigned, std::pair<K, D*>,PSTLSortOrder>::const_iterator i = this->find(ref);
+          PAssert(i != this->end(), psprintf("Item %u not found in collection sz: %u",ref,this->size()));
           return i->second.first;   
       }
 
       D * InternalRemoveResort(unsigned pos) {
           unsigned newpos = pos;
-          unsigned sz = this->size();
+          unsigned sz = (unsigned)this->size();
           D * dataPtr = NULL;
           typename std::map< unsigned, std::pair<K, D*>, PSTLSortOrder >::iterator it = this->find(pos);
           if (it == this->end()) return NULL;
@@ -299,10 +310,12 @@ template <class K, class D> class PSTLDictionary : public PObject,
           
           for (unsigned i = pos+1; i < sz; ++i) {
              typename std::map< unsigned, std::pair<K, D*>, PSTLSortOrder >::iterator j = this->find(i);
-             DictionaryEntry entry =  make_pair(j->second.first,j->second.second) ;
-             this->insert(pair<unsigned, std::pair<K, D*> >(newpos,entry));
-             newpos++;
-             this->erase(j);
+             if (j != this->end()) {
+                 DictionaryEntry entry =  make_pair(j->second.first,j->second.second) ;
+                 this->insert(pair<unsigned, std::pair<K, D*> >(newpos,entry));
+                 newpos++;
+                 this->erase(j);
+             }
           }
 
           return dataPtr;
@@ -326,7 +339,7 @@ template <class K, class D> class PSTLDictionary : public PObject,
       { 
           PWaitAndSignal m(dictMutex);
 
-          unsigned pos = this->size();
+          unsigned pos = (unsigned)this->size();
           DictionaryEntry entry = make_pair(key,obj);
           this->insert(pair<unsigned, std::pair<K, D*> >(pos,entry));
           return true;
@@ -380,6 +393,9 @@ template <class D> class PSTLList : public PObject,
      */
      PSTLList() :disallowDeleteObjects(false) {}
 
+     PSTLList(int /*dummy*/, const PObject * /*c*/)
+        : disallowDeleteObjects(false) { }
+
      ~PSTLList() {  RemoveAll(); }
   //@}
 
@@ -408,7 +424,7 @@ template <class D> class PSTLList : public PObject,
     virtual PINDEX InsertAt(
       PINDEX index,   ///< Index position in collection to place the object.
       D * obj         ///< New object to place into the collection.
-      ) { return InternalSetAt(index,obj,false,true); }
+      ) { return InternalSetAt((unsigned)index,obj,false,true); }
 
     /**Remove the object at the specified ordinal index from the collection.
        If the AllowDeleteObjects option is set then the object is also deleted.
@@ -421,7 +437,7 @@ template <class D> class PSTLList : public PObject,
      */
     virtual D * RemoveAt(
       PINDEX index   ///< Index position in collection to place the object.
-      ) { return InternalRemoveKey(index); }
+      ) { return InternalRemoveKey((unsigned)index); }
 
 
     PBoolean Remove(
@@ -447,7 +463,7 @@ template <class D> class PSTLList : public PObject,
     virtual PBoolean SetAt(
       PINDEX index,   ///< Index position in collection to set.
       D * obj         ///< New value to place into the collection.
-      ) {  return InternalSetAt(index,obj);  }
+      ) {  return InternalSetAt((unsigned)index,obj);  }
     
     /**Set the object at the specified ordinal position to the new value. This
        will overwrite the existing entry. If the AllowDeleteObjects option is
@@ -462,7 +478,7 @@ template <class D> class PSTLList : public PObject,
     virtual PBoolean ReplaceAt(
       PINDEX index,   ///< Index position in collection to set.
       D * obj         ///< New value to place into the collection.
-      ) {  return InternalSetAt(index,obj, true);  }
+      ) {  return InternalSetAt((unsigned)index,obj, true);  }
 
     /**Get the object at the specified ordinal position. If the index was
        greater than the size of the collection then NULL is returned.
@@ -476,10 +492,10 @@ template <class D> class PSTLList : public PObject,
      */
     virtual D * GetAt(
       PINDEX index  ///< Index position in the collection of the object.
-    ) const { return InternalAt(index); }
+    ) const { return InternalAt((unsigned)index); }
 
 
-    D & operator[](PINDEX i) const { return InternalGetAt(i); }
+    D & operator[](PINDEX i) const { return InternalGetAt((unsigned)i); }
 
     /**Search the collection for the specific instance of the object. The
        object pointers are compared, not the values. A simple linear search
@@ -564,6 +580,9 @@ template <class D> class PSTLList : public PObject,
       {
           PWaitAndSignal m(dictMutex);
 
+          if (data == NULL)
+              return false;
+
           typename std::map< unsigned, D* , PSTLSortOrder>::const_iterator i;
           for (i = this->begin(); i != this->end(); ++i) {
             if (i->second == data) {
@@ -612,13 +631,14 @@ template <class D> class PSTLList : public PObject,
 
           PAssert(ref < this->size(), psprintf("Index out of Bounds ref: %u sz: %u",ref,this->size()));
           typename std::map< unsigned, D*, PSTLSortOrder>::const_iterator i = this->find(ref);
-          return i->second;   
+		  if (i != this->end()) return i->second;  
+		  else return NULL;
       };
 
 
       D * InternalRemoveResort(unsigned pos) {
           unsigned newpos = pos;
-          unsigned sz = this->size();
+          unsigned sz = (unsigned)this->size();
           D * dataPtr = NULL;
           typename std::map< unsigned, D*, PSTLSortOrder >::iterator it = this->find(pos);
           if (it == this->end()) return NULL;
@@ -641,7 +661,7 @@ template <class D> class PSTLList : public PObject,
       };
 
       D * InternalRemoveKey(
-            PINDEX pos   ///< Key to look for in the dictionary.
+            unsigned pos   ///< Key to look for in the dictionary.
             )
       {
           PWaitAndSignal m(dictMutex);
@@ -652,62 +672,75 @@ template <class D> class PSTLList : public PObject,
       PINDEX InternalAddKey(
          D * obj         // New object to put into list.
          ) 
-      { 
+      {
           PWaitAndSignal m(dictMutex);
 
-          unsigned pos = this->size();
+          if (obj == NULL)
+              return -1;
+
+          unsigned pos = (unsigned)this->size();
           this->insert(std::pair<unsigned, D*>(pos,obj));
           return pos;
       }
 
       PINDEX InternalSetAt(
-          PINDEX ref,                  ///< Index position in collection to set.
+          unsigned ref,                ///< Index position in collection to set.
           D * obj,                     ///< New value to place into the collection.
           PBoolean replace = false,
           PBoolean reorder = false
           ) 
-      {         
-          if (ref >= GetSize())
+      {     
+          if (obj == NULL)
+              return -1;
+
+          if (ref >= (unsigned)GetSize())
               return InternalAddKey(obj);
 
           PWaitAndSignal m(dictMutex);
 
           if (!reorder) {
               typename std::map< unsigned, D*, PSTLSortOrder>::iterator it = this->find(ref);
-              if (replace)
-                delete it->second;  
-              this->erase(it);
+              if (it != this->end()) {
+                  if (replace) 
+                      delete it->second;  
+                  this->erase(it);
+              }
           } else {
-              //for (typename std::map< unsigned, D*, PSTLSortOrder >::iterator r = this->begin(); r != this->end(); ++r) {
-              //     if (*obj == *(r->second)) {
-              //        InternalRemoveResort(r->first);
-              //        break;
-              //     }
-              //}
-              int sz = this->size();
+              unsigned sz = (unsigned)GetSize();
               if (sz > 0) {
-                  int newpos = sz;
-                  for (int i = sz-1; i >= ref; --i) {
+                  unsigned newpos = sz;
+                  for (unsigned i = sz; i-- > ref; ) {
                      typename std::map< unsigned, D*, PSTLSortOrder >::iterator it = this->find(i);
-                     D* entry =  it->second;
-                     this->insert(std::pair<unsigned, D*>(newpos,entry));
-                     this->erase(it);
-                     newpos--;
+                     if (it != this->end()) {
+                         D* entry =  it->second;
+                         this->insert(std::pair<unsigned, D*>(newpos,entry));
+                         this->erase(it);
+                         newpos--;
+                     }
                   }
               }
           }
           this->insert(std::pair<unsigned, D*>(ref,obj));
           return ref;        
       }
-
 };
 
-#define PSTLLIST(cls, D) typedef PSTLList<D> cls;
-
 #define H323List  PSTLList
-#define H323LIST  PSTLLIST
+#define H323LIST(cls, D) typedef H323List<D> cls;
 
-#endif  // PTLIB_VER < 2110
+#endif  // H323_STLDICTIONARY
+
+#define H323_DECLARELIST(cls, T) \
+  H323LIST(cls##_PTemplate, T); \
+  PDECLARE_CLASS(cls, H323List<T>) \
+  protected: \
+    cls(int dummy, const cls * c) \
+      : H323List<T>(dummy, c) { } \
+  public: \
+    cls() \
+      : H323List<T>() { } \
+    virtual PObject * Clone() const \
+      { return PNEW cls(0, this); } \
 
 
 #ifdef H323_FRAMEBUFFER
@@ -761,7 +794,7 @@ protected:
 
 public:
     H323_FrameBuffer()
-    : PThread(10000, NoAutoDeleteThread), m_threadRunning(false),
+    : PThread(10000, NoAutoDeleteThread, HighestPriority), m_threadRunning(false),
       m_frameMarker(0), m_frameOutput(false), m_frameStartTime(0), 
       m_StartTimeStamp(0), m_calcClockRate(90),
       m_packetReceived(0), m_oddTimeCount(0), m_lateThreshold(5.0), m_increaseBuffer(false),
@@ -781,18 +814,23 @@ public:
         Resume();
     }
 
+	PBoolean IsRunning() 
+	{
+		return m_threadRunning;
+	}
+
     virtual void FrameOut(PBYTEArray & /*frame*/, PInt64 /*receiveTime*/, unsigned /*clock*/, PBoolean /*fup*/, PBoolean /*flow*/) {};
 
     void Main() {
 
         PBYTEArray frame;
         PTimeInterval lastMarker;
-        unsigned delay=0;
+        int delay=0;
         PBoolean fup=false;
 
         while (!m_exit) {
             while (m_frameOutput && m_frameMarker > 0) {
-                if (m_buffer.size() == 0) {
+                if (m_buffer.empty()) {
                     if (m_frameMarker > 0)
                         m_frameMarker--;
                     break;
@@ -811,9 +849,9 @@ public:
                     memcpy(frame.GetPointer(), m_buffer.top().second, m_buffer.top().second.GetSize());
                     unsigned lastTimeStamp = info.m_timeStamp;
                     m_buffer.pop();
-                    if (info.m_marker && m_buffer.size() > 0) { // Peek ahead for next timestamp
+                    if (info.m_marker && !m_buffer.empty()) { // Peek ahead for next timestamp
                         delay = (m_buffer.top().first.m_timeStamp - lastTimeStamp)/(unsigned)m_calcClockRate;
-                        if (delay < 0 || delay > 200 || (lastTimeStamp > m_buffer.top().first.m_timeStamp)) {
+                        if (delay <= 0 || delay > 200 || (lastTimeStamp > m_buffer.top().first.m_timeStamp)) {
                            delay = 0;
                            lastTimeStamp = m_buffer.top().first.m_timeStamp;
                            m_RenderTimeStamp = PTimer::Tick().GetMilliSeconds();
@@ -826,9 +864,8 @@ public:
                       break;
 
                   m_frameCount++;
-                  unsigned diff=0;
                   if (m_lastSequence) {
-                     diff = info.m_sequence - m_lastSequence - 1;
+                     unsigned diff = info.m_sequence - m_lastSequence - 1;
                      if (diff > 0) {
                          PTRACE(5,"RTPBUF\tDetected loss of " << diff << " packets."); 
                          m_lossCount = m_lossCount + diff;
@@ -863,9 +900,10 @@ public:
             }
             PThread::Sleep(5);
         }
-     bufferMutex.Wait();
-        m_buffer.empty();
-     bufferMutex.Signal();
+        bufferMutex.Wait();
+        while (!m_buffer.empty()) 
+           m_buffer.pop();
+        bufferMutex.Signal();
 
         m_threadRunning = false;
     }
@@ -888,7 +926,7 @@ public:
             m_StartTimeStamp = PTimer::Tick().GetMilliSeconds();
         } else if (marker && m_frameOutput) {
             m_calcClockRate = (float)(time - m_frameStartTime)/(PTimer::Tick().GetMilliSeconds() - m_StartTimeStamp);
-            if (m_calcClockRate > 100 || m_calcClockRate < 40) {
+            if (m_calcClockRate > 100 || m_calcClockRate < 40 || (m_calcClockRate == numeric_limits<unsigned int>::infinity( ))) {
                 PTRACE(4,"RTPBUF\tErroneous ClockRate: Resetting...");
                 m_calcClockRate = 90;
                 m_frameStartTime = time;
@@ -907,7 +945,7 @@ public:
 
         bufferMutex.Wait();
         m_packetReceived++;
-        if (m_frameOutput && m_buffer.size() > 0 && info.m_sequence < m_buffer.top().first.m_sequence) {
+        if (m_frameOutput && !m_buffer.empty() && info.m_sequence < m_buffer.top().first.m_sequence) {
             m_oddTimeCount++;
             PTRACE(6,"RTPBUF\tLate Packet Received " << (m_oddTimeCount/m_packetReceived)*100.0 << "%");
             if ((m_oddTimeCount/m_packetReceived)*100.0 > m_lateThreshold) {
@@ -934,6 +972,130 @@ public:
 
 #endif  // H323_FRAMEBUFFER
 
+#if PTLIB_VER < 2130
+
+#define H323UDPSocket PUDPSocket
+
+#else
+class H323UDPSocket : public PUDPSocket
+{
+    PCLASSINFO(H323UDPSocket, PUDPSocket);
+
+    public:
+        H323UDPSocket() {}
+        virtual PBoolean IsAlternateAddress(const Address & /*address*/, WORD /*port*/) { return false; }
+        virtual PBoolean DoPseudoRead(int & /*selectStatus*/) { return false; }
+
+        virtual PBoolean Close() {
+#if 0 //P_QWAVE - QoS not supported yet
+            return PIPSocket::Close();
+#else
+            return PSocket::Close();
+#endif
+        }
+};
+
+#endif // H323UDPSocket
+
+#if PTLIB_VER < 2130
+
+#define H323NatMethod   PNatMethod
+#define H323NatList     PNatList
+#define H323NatStrategy PNatStrategy
+
+#else
+
+class H323NatMethod  : public PNatMethod
+{
+    PCLASSINFO(H323NatMethod,PNatMethod);
+
+    public:
+        H323NatMethod() : PNatMethod(0) {}
+
+        virtual PString GetServer() const { return PString(); }
+        virtual PCaselessString GetMethodName() const { return "dummy"; }
+
+        PString GetName() { return GetMethodName(); }
+
+        virtual bool IsAvailable(
+          const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny()  ///< Interface to see if NAT is available on
+        ) = 0;
+
+    protected:
+      struct PortInfo {
+      PortInfo(WORD port = 0)
+      : basePort(port), maxPort(port), currentPort(port) {}
+          PMutex mutex;
+          WORD   basePort;
+          WORD   maxPort;
+          WORD   currentPort;
+    } singlePortInfo, pairedPortInfo;
+
+    virtual PNATUDPSocket * InternalCreateSocket(Component /*component*/, PObject * /*context*/)  { return NULL; }
+    virtual void InternalUpdate() {};
+};
+
+H323LIST(H323NatList, PNatMethod);
+
+class H323NatStrategy : public PObject
+{
+public :
+
+    H323NatStrategy() {};
+    ~H323NatStrategy() { natlist.RemoveAll(); }
+
+  void AddMethod(PNatMethod * method) { natlist.Append(method); }
+
+  PNatMethod * GetMethod(const PIPSocket::Address & address = PIPSocket::GetDefaultIpAny()) {
+      for (PINDEX i=0; i < natlist.GetSize(); ++i) {
+        if (natlist[i].IsAvailable(address))
+          return &natlist[i];
+      }
+      return NULL;
+  }
+
+  PNatMethod * GetMethodByName(const PString & name) {
+      for (PINDEX i=0; i < natlist.GetSize(); ++i) {
+        if (natlist[i].GetMethodName() == name) {
+          return &natlist[i];
+        }
+      }
+      return NULL;
+  }
+
+  PBoolean RemoveMethod(const PString & meth) {
+      for (PINDEX i=0; i < natlist.GetSize(); ++i) {
+        if (natlist[i].GetMethodName() == meth) {
+          natlist.RemoveAt(i);
+          return true;
+        }
+      }
+      return false;
+  }
+
+  void SetPortRanges( WORD portBase, WORD portMax = 0, WORD portPairBase = 0, WORD portPairMax = 0) {
+      for (PINDEX i=0; i < natlist.GetSize(); ++i)
+        natlist[i].SetPortRanges(portBase, portMax, portPairBase, portPairMax);
+  }
+
+  H323NatList & GetNATList() {  return natlist; }
+
+  PNatMethod * LoadNatMethod(const PString & name) {
+    PPluginManager * pluginMgr = &PPluginManager::GetPluginManager();
+    return (PNatMethod *)pluginMgr->CreatePlugin(name, "PNatMethod");
+  }
+
+  static PStringArray GetRegisteredList() {
+    PPluginManager * pluginMgr = &PPluginManager::GetPluginManager();
+    return pluginMgr->GetPluginsProviding("PNatMethod", false);
+  }
+
+private:
+  H323NatList natlist;
+};
+
+#endif  // H323_NATMETHOD
+
 #endif // _PTLIB_EXTRAS_H
 
 
