@@ -26,7 +26,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $ Id $
+ * $Id: transports.h,v 1.24 2014/01/08 06:25:15 shorne Exp $
  *
  */
 
@@ -41,6 +41,9 @@
 #include <ptlib/sockets.h>
 #include "ptlib_extras.h"
 
+#ifdef H323_TLS
+#include <ptclib/pssl.h>
+#endif
 
 class H225_Setup_UUIE;
 class H225_TransportAddress;
@@ -57,18 +60,15 @@ class H323Listener;
 class H323Transport;
 class H323Gatekeeper;
 
-
-
 ///////////////////////////////////////////////////////////////////////////////
 
 /**String representation of a transport address.
  */
-
 class H323TransportAddress : public PString
 {
   PCLASSINFO(H323TransportAddress, PString);
   public:
-    H323TransportAddress() { m_version = 4; }
+    H323TransportAddress() : m_version(4), m_tls(false) {}
     H323TransportAddress(const char *);
     H323TransportAddress(const PString &);
     H323TransportAddress(const H225_TransportAddress &);
@@ -100,6 +100,10 @@ class H323TransportAddress : public PString
       const char * proto = "tcp"
     ) const;
 
+    /**Get Port Number.
+      */
+    WORD GetPort() const;
+
     /**Get the IP Version
        6 = IPv6  4 = IPv4
       */
@@ -141,11 +145,16 @@ class H323TransportAddress : public PString
       H323EndPoint & endpoint   ///<  Endpoint object for transport creation.
     ) const;
 
+    /**Set the transport Address security settings
+      */
+    void SetTLS(PBoolean isTLS);
+
   protected:
     void Validate();
 
   private:
-    unsigned m_version;
+    unsigned              m_version;
+    PBoolean              m_tls;
 };
 
 
@@ -186,6 +195,51 @@ PDECLARE_ARRAY(H323TransportAddressArray, H323TransportAddress)
     );
 };
 
+///////////////////////////////////////////////////////////////////////////////
+/**Transport Security Information.
+ */
+
+class H323TransportSecurity {
+public:
+
+    H323TransportSecurity(H323EndPoint * ep = NULL);
+
+    enum Method {
+        e_unsecure,
+        e_tls,
+        e_ipsec     // not supported YET
+    };
+    static PString MethodAsString(Method meth);
+
+    enum Policy {
+        e_nopolicy,             // no policy
+        e_reqTLSMediaEncHigh,   // require TLS to offer high media encryption
+        e_reqTLSMediaEncAll     // require TLS to offer ALL media encryption
+    };
+    static PString PolicyAsString(Policy policy);
+
+    PBoolean HasSecurity();
+
+    void EnableTLS(PBoolean enable); 
+    PBoolean IsTLSEnabled() const;
+
+    void SetRemoteTLSAddress(const H323TransportAddress & address);
+    H323TransportAddress GetRemoteTLSAddress();
+
+    void EnableIPSec(PBoolean enable); 
+    PBoolean IsIPSecEnabled();
+
+    void SetMediaPolicy(Policy policy);
+    Policy GetMediaPolicy() const;
+
+    void Reset();
+
+protected:
+    int m_securityMask;
+    int m_policyMask;
+    H323TransportAddress m_remoteTLSAddress;
+};
+
 
 /**This class describes a "listener" on a transport protocol.
    A "listener" is an object that listens for incoming connections on the
@@ -209,7 +263,8 @@ class H323Listener : public PThread
     /**Create a new listener.
      */
     H323Listener(
-      H323EndPoint & endpoint      ///<  Endpoint instance for channel
+      H323EndPoint & endpoint,     ///<  Endpoint instance for channel
+      H323TransportSecurity::Method security = H323TransportSecurity::e_unsecure
     );
   //@}
 
@@ -240,20 +295,45 @@ class H323Listener : public PThread
       */
     virtual H323TransportAddress GetTransportAddress() const = 0;
 
+    /**Set the local transport address on which this listener may be accessed.
+      */
+    virtual void SetTransportAddress(const H323TransportAddress & address) = 0;
+
+    /**Create Transport.
+      */
+    virtual H323Transport * CreateTransport(const PIPSocket::Address & address) = 0;
+
     /**Set up a transport address PDU for bidirectional logical channels.
       */
     virtual PBoolean SetUpTransportPDU(
       H245_TransportAddress & pdu,         ///<  Transport addresses listening on
       const H323Transport & associatedTransport ///<  Associated transport for precendence and translation
     ) = 0;
+
+    /**Get Signalling Security.
+      */
+    H323TransportSecurity::Method GetSecurity();
+
+    /**Listener Type as string.
+      */
+    PString TypeAsString() const;
   //@}
 
   protected:
     H323EndPoint & endpoint;  /// Endpoint that owns the listener.
+    H323TransportSecurity::Method   m_security;
 };
 
-
-H323LIST(H323ListenerList, H323Listener);
+H323_DECLARELIST(H323ListenerList, H323Listener)
+#ifdef DOC_PLUS_PLUS
+{
+#endif
+  public:
+	  H323Listener * GetListener() const;
+#ifdef H323_TLS
+	  H323Listener * GetTLSListener() const;
+#endif
+};
 
 
 /** Return a list of transport addresses corresponding to a listener list
@@ -285,6 +365,23 @@ void H323SetTransportAddresses(
    A "transport" is an object that listens for incoming connections on the
    particular transport.
  */
+#ifdef H323_TLS
+class H323Transport : public PSSLChannel
+{
+  PCLASSINFO(H323Transport, PSSLChannel);
+
+  public:
+  /**@name Construction */
+  //@{
+    /**Create a new transport channel.
+     */
+    H323Transport(H323EndPoint & endpoint,
+                  PSSLContext * context = NULL,   ///< Context for SSL channel
+                  PBoolean autoDeleteContext = false  ///< Flag for context to be automatically deleted.
+    );
+    ~H323Transport();
+  //@}
+#else
 class H323Transport : public PIndirectChannel
 {
   PCLASSINFO(H323Transport, PIndirectChannel);
@@ -297,7 +394,7 @@ class H323Transport : public PIndirectChannel
     H323Transport(H323EndPoint & endpoint);
     ~H323Transport();
   //@}
-
+#endif
   /**@name Overrides from PObject */
   //@{
     virtual void PrintOn(
@@ -334,6 +431,40 @@ class H323Transport : public PIndirectChannel
       const H323TransportAddress & address
     ) { return SetRemoteAddress(address) && Connect(); }
 
+
+    /**Low level read from the channel.
+     */
+    virtual PBoolean Read(
+      void * buf,   ///< Pointer to a block of memory to receive the read bytes.
+      PINDEX len    ///< Maximum number of bytes to read into the buffer.
+    );
+
+    /**Low level write to the channel. 
+     */
+    virtual PBoolean Write(
+      const void * buf, ///< Pointer to a block of memory to write.
+      PINDEX len        ///< Number of bytes to write.
+    );
+
+    /**This callback is executed when the OnOpen() function is called with
+       open channels. It may be used by descendent channels to do any
+       handshaking required by the protocol that channel embodies.
+
+       The default behaviour is to simply return TRUE.
+
+       @return
+       Returns TRUE if the protocol handshaking is successful.
+     */
+    virtual PBoolean OnSocketOpen();
+
+    /**Determine if the channel is currently open and read and write operations
+       can be executed on it.
+
+       @return
+       true if the channel is open.
+     */
+    virtual PBoolean IsOpen() const;
+
     /**Close the channel.
       */
     virtual PBoolean Close();
@@ -421,6 +552,15 @@ class H323Transport : public PIndirectChannel
     virtual PBoolean WritePDU(
       const PBYTEArray & pdu  ///<  PDU to write
     ) = 0;
+
+    /**Write a protocol data unit from the transport.
+       This will write using the transports mechanism for PDU boundaries, for
+       example UDP is a single Write() call, while for TCP there is a TPKT
+       header that indicates the size of the PDU.
+      */
+    virtual PBoolean WriteSignalPDU(
+      const H323SignalPDU & /*pdu*/  /// PDU to write
+    ) { return false; }
   //@}
 
   /**@name Signalling Channel */
@@ -476,8 +616,27 @@ class H323Transport : public PIndirectChannel
    /**Whether the RAS is tunnelled AKA H.460.17
       */
     virtual PBoolean IsRASTunnelled()  { return false; }
-  //@}
 
+   /**Initialise Security
+      */
+    virtual PBoolean InitialiseSecurity(const H323TransportSecurity * /*security*/) { return false; }
+
+    /**Finalise Transport Security.
+      */
+    virtual PBoolean FinaliseSecurity(PSocket * /*socket*/) { return false; }
+
+   /**Whether the Transport is secured
+      */
+    PBoolean IsTransportSecure();
+
+    /**Do SSL Connect handshake
+      */
+    virtual PBoolean SecureConnect() { return false; }
+
+    /**Do SSL Accept handshake
+      */
+    virtual PBoolean SecureAccept() { return false; }
+  //@}
 
   /**@name Member variable access */
   //@{
@@ -493,7 +652,7 @@ class H323Transport : public PIndirectChannel
 
     /**Wait for associated thread to terminate.
       */
-    void CleanUpOnTermination();
+    virtual void CleanUpOnTermination();
 
     /**Get Error Code
        Default calls PChannel::GetErrorCode(ErrorGroup group)
@@ -502,9 +661,22 @@ class H323Transport : public PIndirectChannel
   //@}
 
   protected:
+    /**This callback is executed when the Open() function is called with
+       open channels. It may be used by descendent channels to do any
+       handshaking required by the protocol that channel embodies.
+
+       The default behaviour is to simply return true.
+
+       @return
+       Returns true if the protocol handshaking is successful.
+     */
+    virtual PBoolean OnOpen();
+
     H323EndPoint & endpoint;    /// Endpoint that owns the listener.
     PThread      * thread;      /// Thread handling the transport
     PBoolean canGetInterface;
+
+    PBoolean    m_secured;     /// Whether the channel is secure.
 };
 
 
@@ -522,10 +694,15 @@ class H323TransportIP : public H323Transport
   public:
     /**Create a new transport channel.
      */
+
     H323TransportIP(
       H323EndPoint & endpoint,    ///<  H323 End Point object
       PIPSocket::Address binding, ///<  Local interface to use
       WORD remPort                ///<  Remote port to use
+#ifdef H323_TLS
+      ,PSSLContext * context = NULL,   ///< Context for SSL channel
+      PBoolean autoDeleteContext = false  ///< Flag for context to be automatically deleted.
+#endif
     );
 
     /**Get the transport dependent name of the local endpoint.
@@ -582,7 +759,8 @@ class H323ListenerTCP : public H323Listener
       H323EndPoint & endpoint,    ///<  Endpoint instance for channel
       PIPSocket::Address binding, ///<  Local interface to listen on
       WORD port,                  ///<  TCP port to listen for connections
-      PBoolean exclusive = FALSE      ///<  Fail if listener port in use
+      PBoolean exclusive = FALSE,    ///<  Fail if listener port in use
+      H323TransportSecurity::Method security = H323TransportSecurity::e_unsecure ///< TransportSecurity
     );
 
     /** Destroy the listener thread.
@@ -608,6 +786,15 @@ class H323ListenerTCP : public H323Listener
       */
     virtual H323TransportAddress GetTransportAddress() const;
 
+    /**Set the local transport address on which this listener may be accessed.
+        Default does nothing...
+      */
+    virtual void SetTransportAddress(const H323TransportAddress & address);
+
+    /**Create Transport.
+      */
+    virtual H323Transport * CreateTransport(const PIPSocket::Address & address);
+
     /**Set up a transport address PDU for bidirectional logical channels.
       */
     virtual PBoolean SetUpTransportPDU(
@@ -617,7 +804,6 @@ class H323ListenerTCP : public H323Listener
 
     WORD GetListenerPort() const { return listener.GetPort(); }
 
-
   protected:
     /**Handle incoming H.323 connections and dispatch them in new threads
        based on the H323Transport class. This is defined in the descendent
@@ -635,6 +821,51 @@ class H323ListenerTCP : public H323Listener
     PBoolean exclusiveListener;
 };
 
+//////////////////////////////////////////////////////////////////////////////////
+// Transport classes for TLS
+
+#ifdef H323_TLS
+
+/**This class manages H323 connections using TLS transport.
+ */
+class H323ListenerTLS : public H323ListenerTCP
+{
+  PCLASSINFO(H323ListenerTLS, H323ListenerTCP);
+
+  public:
+    /**Create a new listener for the TCP/IP protocol.
+     */
+    H323ListenerTLS(
+      H323EndPoint & endpoint,    ///<  Endpoint instance for channel
+      PIPSocket::Address binding, ///<  Local interface to listen on
+      WORD port,                  ///<  TCP port to listen for connections
+      PBoolean exclusive = FALSE  ///<  Fail if listener port in use
+    );
+
+    /** Destroy the listener thread.
+      */
+    ~H323ListenerTLS();
+
+    /**Create Transport.
+      */
+    virtual H323Transport * CreateTransport(const PIPSocket::Address & address);
+
+    /**Set up a transport address PDU for bidirectional logical channels. return false (Not used)
+      */
+    virtual PBoolean SetUpTransportPDU(
+      H245_TransportAddress & pdu,         ///<  Transport addresses listening on
+      const H323Transport & associatedTransport ///<  Associated transport for precendence and translation
+    );
+
+    /**Set the local transport address on which this listener may be accessed.
+        Default sets the local address
+      */
+    virtual void SetTransportAddress(const H323TransportAddress & address);
+};
+
+#endif // H323_TLS
+
+/////////////////////////////////////////////////////////////////////////////////
 
 /**This class represents a particular H323 transport using TCP/IP.
  */
@@ -649,6 +880,10 @@ class H323TransportTCP : public H323TransportIP
       H323EndPoint & endpoint,    ///<  H323 End Point object
       PIPSocket::Address binding = PIPSocket::GetDefaultIpAny(), ///<  Local interface to use
       PBoolean listen = FALSE         ///<  Flag for need to wait for remote to connect
+#ifdef H323_TLS
+      ,PSSLContext * context = NULL,   ///< Context for SSL channel
+      PBoolean autoDeleteContext = false  ///< Flag for context to be automatically deleted.
+#endif
     );
 
     /**Destroy transport channel.
@@ -668,6 +903,17 @@ class H323TransportTCP : public H323TransportIP
       */
     virtual PBoolean Connect();
 
+    /**This callback is executed when the OnOpen() function is called with
+       open channels. It may be used by descendent channels to do any
+       handshaking required by the protocol that channel embodies.
+
+       The default behaviour is to simply return TRUE.
+
+       @return
+       Returns TRUE if the protocol handshaking is successful.
+     */
+    virtual PBoolean OnSocketOpen();
+
     /**Close the channel.
       */
     virtual PBoolean Close();
@@ -717,16 +963,43 @@ class H323TransportTCP : public H323TransportIP
       */
     virtual PBoolean IsListening() const;
 
+#if PTLIB_VER >= 2130
+   /** Read a single character from the channel. 
+        Casting issue in PTLIB
+     */
+    virtual int ReadChar();
+
+   /** Read len bytes into the buffer from the channel.
+        Casting issue in PTLIB
+     */
+    virtual PBoolean ReadBlock(void * buf, PINDEX len);
+#endif
+
+    /**Initialise Transport Security.
+      */
+    virtual PBoolean InitialiseSecurity(const H323TransportSecurity * security);
+
+    /**Finalise Transport Security.
+      */
+    virtual PBoolean FinaliseSecurity(PSocket * socket);
+
+    /**Do SSL Connect handshake
+      */
+    virtual PBoolean SecureConnect();
+
+    /**Do SSL Accept handshake
+      */
+    virtual PBoolean SecureAccept();
 
   protected:
     /**This callback is executed when the Open() function is called with
        open channels. It may be used by descendent channels to do any
        handshaking required by the protocol that channel embodies.
 
-       The default behaviour is to simply return TRUE.
+       The default behaviour is to simply return true.
 
        @return
-       Returns TRUE if the protocol handshaking is successful.
+       Returns true if the protocol handshaking is successful.
      */
     virtual PBoolean OnOpen();
 
